`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company:				jc-21.ac Japan
// Engineer:			Takeshi Sakato with Mikata Masahito
// 
// Create Date:		13:15:02 12/10/2013 
// Design Name:		RangFinderForAVES
// Module Name:		RangFinderForAVES 
// Project Name:		RangFinderForAVES
// Target Devices:	Xilinx Spartan3E
// Tool versions:		ISE 8.2i / ISE 10.1
// Description: 
// (Lq)
//
// Dependencies: 
// (ˑ)
//
// Revision:			0.01 - File Created			13:15:02 05/27/2012
// Revision:			0.02 - File Update 			07/28/2012
// Revision:			0.03 - File Update 			12/16/2012
// Revision:			0.04 - File Update 			12/16/2012
// Revision:			0.05 - File Update 			01/20/2013
// Revision:			0.06 - File Update 			01/24/2013
// Revision:			1.01 - File Update 			12/10/2013
// Revision:			2013 edition 					18/Feb/2014
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module RangFinderCanForCO_works(
	CLK,
	DATA_IN,
	TRG_EN, 
	DATA_EN,
	TRG,
	//PLS_1mm,	//2013 edition
	LED,
	COM,
	SEG7,
	//TP_TRG,	//2013 edition
	TX_SELECT,
	RxD,
	TxD,
	SCK,
	SO,
	CS
	);

	input  CLK;								// Input Master clock
	input	 TX_SELECT;							// Input Transmit Data Select
	input  DATA_IN;						// Input Ultrasonic sensor
	input  RxD;								// RS232C Mް
	output TRG_EN;							// Sensing trigger enable
	output DATA_EN;						// Data enable
	output TRG;								// Trigger						
	
//2013 edition
	//output PLS_1mm;
	output SCK;								//SPI Clock
	output SO;								//SPI MOSI
	output CS;								//SPI Slave select

////////// 2013 Version Update
//	output [8:1]LED;						// 2012 Version
	output [4:1]LED;						// Monitor mm 2013 Version

	output [4:1]COM;						// Select 7segLED
	output [8:1]SEG7;						// Display
//	output TP_TRG;							// ضށE
	output TxD;								// RS232C Mް

	parameter data1_start = 800000;			// 24.0002 msec
	parameter data2_start = 821000;			// 24.6302 msec
	parameter data3_start = 842000;			// 25.2603 msec
	parameter data4_start = 863000;			// 25.8903 msec
	parameter data5_start = 884000;			// 26.5203 msec
	parameter data6_start = 905000;			// 27.1503 msec

	parameter bit3  = 2604;					//  78.12 usec --- Tx Reset
	parameter bit6  = 5208;					// 156.24 usec --- Tx Data Write
	parameter bit9  = 7812;					// 234.36 usec --- Hold
	parameter bit24 = 20832;				// 624.97 usec --- Tx Start

	reg [20:0]master_counter = 0;		// Internal master counter initialize
	reg trg_en_flg = 1;					// Sensing trigger enable flag
	reg trg_flg = 0;						// Sensing trigger flag
	reg data_en_flg = 1;					// Range data enable flag

	reg [7:0]count_1mm = 0;				// 1mm counter initialize
	reg count_1mm_flg = 0;				// 1mm counter flag

	reg TX_WR = 0;							// Enable transmit flag
	reg TX_EN = 0;							// Enable transmit flag
	reg TX_RST = 1;							// Reset transmit module
	reg MODE = 1;

	reg [5:0]Tx_Data_Cnt = 6'b111110;		// Tx Data Select Pulse

	reg [15:0]hold_renge_dec = 0;			// Tx & Display Hold Data(Dec)

	wire [15:0]renge_dec;					// Count Renge Data (Dec)
	wire [15:0]renge_bin;					// Count Renge Data (Bin)

	wire [15:0]hold_renge_sel;
	wire Count_Data_Enable;
	wire ComScan;
	wire [6:1]ComSelect;
	wire [7:0]Send_Data;

	wire TX_CK;
	wire RX_CK;

//2013 edition
	wire PLS_1mm;							//delete ucf
	
	wire clk8m;								//SCK 8.33MHz
	wire cs20;								//SPI sleve select
	wire so20;								//SPI MOSI
	wire [39:0] rengedt;					//range data SOF 1byte + DATA 4byte(8+32=40bit)

////////// Assign I/O Pulse
	assign 	TRG_EN = trg_en_flg;
	assign 	TRG = trg_flg;
	assign 	DATA_EN = data_en_flg;
	
//2013 edition delete
//	assign	TP_TRG = trg_en_flg & trg_flg;

	assign	LED[4:1] = ~hold_renge_dec[3:0];

////////// 2013 Version Deleted
//	assign	LED[8:5] = {MODE, TX_SELECT, 2'b11};

	assign	COM = ComSelect[4:1];
	
//2013 edition SPI assign 

	assign SCK = clk8m;
	assign CS = cs20;
	assign SO = so20;

////////// Generating Transmit Sel Flg
	always @(posedge TX_SELECT)
	begin
		MODE <= MODE ^ 1;
	end

////////// Module Session
	// 7segLED common select PLS
	Gen_Com_Scan U0(CLK, ComScan);

	// RS232C Mۯ 38400bps
	RS232C_CLKGEN U1( CLK, RST, TX_CK, RX_CK );

	// Send_Data_Slect
	Send_Data_Slect U2(Tx_Data_Cnt, MODE, hold_renge_dec, Send_Data);

	// RS232C ްM data bit:8 stop bit:2 parity:none x-on/off:none
	RS232C_TX     U3( TX_CK, RST, TX_EN, TX_WR, Send_Data, TxD );

	// Generating 1mm Pulse(33.333MHz -> 99.799KHz(173.609375KHz))
	Gen_1mm_Pulse U4(CLK, PLS_1mm);

	// Count_Renge
	Count_Renge U5(master_counter, data_en_flg, DATA_IN, PLS_1mm, renge_dec, renge_bin);

	// Didplay Renge Data
	Disp_Counter U6(ComScan, {4'h0,hold_renge_dec[15:4]}, SEG7, ComSelect);

	// Generat SCK
	clk8m_gen U10(CLK, clk8m);
	
	// MCP2515 register set
	regset_2515 U20(clk8m, rengedt, cs20, so20);
	
	// Colect range data
	range U30(CLK, rengedt, hold_renge_dec, MODE);

////////// Generating Triger  & Enable Pulse
	always @(posedge CLK)
	begin
		if(master_counter == 1333320) begin			// 25Hz(40 msec)
			master_counter <= 0;
			trg_flg <= 0;
			trg_en_flg <= 1;
			data_en_flg <= 1;
			Tx_Data_Cnt <= 6'b111110;
		end
	///////////////////////////
	///// triger Sequence
		else begin
			/// triger enable
			if(master_counter == 667) begin			// 20 usec
				trg_en_flg <= 0;
			end
				/// triger
				if(master_counter == 1167) begin		// 35 usec
					trg_flg <= 1;
				end
				if(master_counter == 1333) begin		// 40 usec(39.99usec)
					trg_flg <= 0;
				end
			if(master_counter == 1834) begin			// 55 usec
				trg_en_flg <= 1;
			end

	///////////////////////////
	///// Data Count Sequence
	///// Data Count Start
			if(master_counter == 18000) begin		// 540 usec
				data_en_flg <=  0;
			end

	///// Count Data Save
			if(master_counter == 700000) begin		// 21 msec
				hold_renge_dec <= renge_dec;
			end

	///// Data Count Stop
			if(master_counter == 705000) begin		// 21.15 msec
				data_en_flg <= 1;
			end

	///////////////////////////
	///// RS232C Sequence Data 1 - 24.0002 msec /////
			if(master_counter == data1_start) begin			// Sequence Start
			///// transmit reset Stop
				TX_RST = 0;							// Reset transmit module
			///// Data Select Tx Data Type [B] or [D]
				Tx_Data_Cnt <= 6'b111110;
			end
			///// transmit data set
			if(master_counter == (data1_start + bit3)) begin			// 130usec
				TX_WR <=  1;
			end
			if(master_counter == (data1_start + bit6)) begin			// 260usec
				TX_WR <=  0;
			end
			if(master_counter == (data1_start + bit9)) begin			// 312usec
				TX_EN <=  1;
			end
			if(master_counter == (data1_start + bit24)) begin			// 312usec
			///// transmit reset
				TX_RST = 1;							// Reset transmit module
				TX_EN <=  0;
			end

	///// RS232C Sequence Data 2 - 24.6302 msec /////
			///// transmit reset
			if(master_counter == data2_start) begin			// Sequence Start
			///// transmit reset Stop
				TX_RST = 0;							// Reset transmit module
			///// Data Select Tx Data Bit15-12
				Tx_Data_Cnt <= 6'b111101;
			end
			///// transmit data set
			if(master_counter == (data2_start + bit3)) begin			// 130usec
				TX_WR <=  1;
			end
			if(master_counter == (data2_start + bit6)) begin			// 260usec
				TX_WR <=  0;
			end
			if(master_counter == (data2_start + bit9)) begin			// 312usec
				TX_EN <=  1;
			end
			if(master_counter == (data2_start + bit24)) begin			// 312usec
			///// transmit reset
				TX_RST = 1;							// Reset transmit module
				TX_EN <=  0;
			end

	///// RS232C Sequence Data 3 - 25.2603 msec /////
			///// transmit reset
			if(master_counter == data3_start) begin			// Sequence Start
			///// transmit reset Stop
				TX_RST = 0;							// Reset transmit module
			///// Data Select Tx Data Bit11-8
				Tx_Data_Cnt <= 6'b111011;
			end
			///// transmit data set
			if(master_counter == (data3_start + bit3)) begin			// 130usec
				TX_WR <=  1;
			end
			if(master_counter == (data3_start + bit6)) begin			// 260usec
				TX_WR <=  0;
			end
			if(master_counter == (data3_start + bit9)) begin			// 312usec
				TX_EN <=  1;
			end
			if(master_counter == (data3_start + bit24)) begin			// 312usec
				TX_RST = 1;							// Reset transmit module
				TX_EN <=  0;
			end

	///// RS232C Sequence Data 4 - 25.8903 msec /////
			///// transmit reset
			if(master_counter == data4_start) begin			// Sequence Start
			///// transmit reset Stop
				TX_RST = 0;							// Reset transmit module
			///// Data Select Tx Data Bit7-4
				Tx_Data_Cnt <= 6'b110111;
			end
			///// transmit data set
			if(master_counter == (data4_start + bit3)) begin			// 130usec
				TX_WR <=  1;
			end
			if(master_counter == (data4_start + bit6)) begin			// 260usec
				TX_WR <=  0;
			end
			if(master_counter == (data4_start + bit9)) begin			// 312usec
				TX_EN <=  1;
			end
			if(master_counter == (data4_start + bit24)) begin			// 312usec
				TX_RST = 1;							// Reset transmit module
				TX_EN <=  0;
			end

	///// RS232C Sequence Data 5 - 26.5203 msec /////
			///// transmit reset
			if(master_counter == data5_start) begin			// Sequence Start
			///// transmit reset Stop
				TX_RST = 0;							// Reset transmit module
			///// Data Select Tx Data Bit3-0
				Tx_Data_Cnt <= 6'b101111;
			end
			///// transmit data set
			if(master_counter == (data5_start + bit3)) begin			// 130usec
				TX_WR <=  1;
			end
			if(master_counter == (data5_start + bit6)) begin			// 260usec
				TX_WR <=  0;
			end
			if(master_counter == (data5_start + bit9)) begin			// 312usec
				TX_EN <=  1;
			end
			if(master_counter == (data5_start + bit24)) begin			// 312usec
				TX_RST = 1;							// Reset transmit module
				TX_EN <=  0;
			end

	///// RS232C Sequence Data 6 - 27.1503 msec/////
			///// transmit reset
			if(master_counter == data6_start) begin			// Sequence Start
			///// transmit reset Stop
				TX_RST = 0;							// Reset transmit module
			///// Data Select Tx Data End Mark [E]
				Tx_Data_Cnt <= 6'b011111;
			end
			///// transmit data set
			if(master_counter == (data6_start + bit3)) begin			// 130usec
				TX_WR <=  1;
			end
			if(master_counter == (data6_start + bit6)) begin			// 260usec
				TX_WR <=  0;
			end
			if(master_counter == (data6_start + bit9)) begin			// 312usec
				TX_EN <=  1;
			end
			if(master_counter == (data6_start + bit24)) begin			// 312usec
				TX_RST = 1;							// Reset transmit module
				TX_EN <=  0;
			end

	///// RS232C Sequence end /////
	///////////////////////////////

			master_counter <= master_counter + 1;

		end
	end

endmodule
