////////////////////////////////////////////
// 			RangeFinder Interface			//
//		2013 edition by Mikata Masahito		//
// 	Module Name:    range		 			//
//		Instance Name:  U30						//
////////////////////////////////////////////
// 			Revision record
// 	ver 1.0 New RTL 2014.02.21

module range(GCLK, rengedt, hold_renge_dec, MODE);

input GCLK;							//Global clock
//RangeFinder colect data
//[15:12] = Range data 1000mm
//[11:8]  = Range data  100mm
//[7:4]   = Range data   10mm
//[3:0]   = Range data    1mm

input [15:0] hold_renge_dec;

//BCD/ASCII select

input MODE;

//Range data
//[39:32] = Delimit 0x42 = BCD / 0x44 = ASCII
//[31:24] = Range data 1000mm
//[23:16] = Range data  100mm
//[15:8] = Range data    10mm
//[7:0] = Range data      1mm

output [39:0] rengedt;								

reg [39:0] rangedt_buf = 0;						//Range data buffer

//*******************************//
//			Colect range data			//
//*******************************//

always @(posedge GCLK)
begin
	if(MODE) begin										//ASCII
		rangedt_buf <= {8'b01000100,				//Start of frame delimit 0x44
							 4'b0011,hold_renge_dec[15:12],
							 4'b0011,hold_renge_dec[11:8],
							 4'b0011,hold_renge_dec[7:4],
							 4'b0011,hold_renge_dec[3:0]};
	end
	
	else begin											//BCD
		rangedt_buf <= {8'b01000010,				//Start of frame delimit 0x42
							 4'b0000,hold_renge_dec[15:12],
							 4'b0000,hold_renge_dec[11:8],
							 4'b0000,hold_renge_dec[7:4],
							 4'b0000,hold_renge_dec[3:0]};
	end
end

// Range data assign

assign rengedt[31:0] = rangedt_buf[31:0];

endmodule
