////////////////////////////////////////////
// 			MCP2515 Register set				//
//		2013 edition by Mikata Masahito		//
// 	Module Name:    regset_2515 			//
//		Instance Name:  U20						//
////////////////////////////////////////////
// 			Revision record
// 	ver 1.0 New RTL 2014.02.21

module regset_2515(clk8m, rengedt, cs20, so20);

input clk8m;									//SCK GEN

//Range data
//[39:32] = Delimit 0x42 = BCD / 0x44 = ASCII
//[31:24] = Range data 1000mm
//[23:16] = Range data  100mm
//[15:8] = Range data    10mm
//[7:0] = Range data      1mm

input [39:0] rengedt;

output cs20;			//SPI Sleve select
output so20;			//SPI MOSI

wire cs1;				//SPI Sleve select status
wire so1;				//SPI MOSI status

reg RST = 1;			//SPI Module Reset
reg length_f = 0;		//SPI data length status 0=3byte 1=1byte

reg [11:0]framecou = 0;		//clk8m counter
reg [7:0]addcou = 0;			//frame counter
reg [7:0]spicom;				//MCP2515 command
reg [7:0]spiadd;				//MCP2515 register address
reg [7:0]spidat;				//MCP2515 register data

//****************************//
//		MCP2515 Parameters		//
//****************************//

//Write to register command
parameter spicom02 = 8'b00000010;

//CANCTRL 0x0F
//Out of config
parameter spiadd0F = 8'b00001111;	//Reg Address 0x28 
parameter spidat0F = 8'b00000100;	//MCP2515 Normal mode
parameter spidat0FC = 8'b10000100;	//MCP2515 Config mode

//CNF3 0x28
//in oder to 500Kbps
parameter spiadd28 = 8'b00101000;	//Reg Address 0x28 
parameter spidat28 = 8'b00000010;	//CAN bit time PS2=3TQ

//CNF2 0x29
//in oder to 500Kbps
parameter spiadd29 = 8'b00101001;	//Reg Address 0x29 
parameter spidat29 = 8'b10010010;	//CAN bit time PS1=PRP=3TQ

//TXB0SIDH 0x31
parameter spiadd31 = 8'b00110001;	//Reg Address 0x31 
parameter spidat31 = 8'b00000000;	//CAN Base ID 10-3 = 0x00

//TXB0SIDL 0x32
parameter spiadd32 = 8'b00110010;	//Reg Address 0x32 
parameter spidat32 = 8'b00101000;	//CAN Base ID 2-0 = 001
												//Ext ID enable = 1,Ext ID17-16 = 00
//TXB0EID8 0x33
parameter spiadd33 = 8'b00110011;	//Reg Address 0x33 
parameter spidat33 = 8'b00000000;	//CAN Ext ID 15-8 = 0x00

//TXB0EID0 0x34
parameter spiadd34 = 8'b00110100;	//Reg Address 0x34 
parameter spidat34 = 8'b00000000;	//CAN Ext ID 7-0 = 0x00

//TXB0DLC 0x35
parameter spiadd35 = 8'b00110101;	//Reg Address 0x35 
parameter spidat35 = 8'b00001000;	//CAN is Data frame. length 8byte
	
//CAN Data frame

//TXB0D0 0x36 Data buffer byte0
parameter spiadd36 = 8'b00110110;	//Reg Address 0x36 

//TXB0D1 0x37 Data buffer byte1
parameter spiadd37 = 8'b00110111;	//Reg Address 0x37

//TXB0D2 0x38 Data buffer byte2
parameter spiadd38 = 8'b00111000;	//Reg Address 0x38 

//TXB0D3 0x39 Data buffer byte3
parameter spiadd39 = 8'b00111001;	//Reg Address 0x39

//TXB0D4 0x3A Data buffer byte4
parameter spiadd3A = 8'b00111010;	//Reg Address 0x3A

//TXB0D5 0x3B Data buffer byte5
parameter spiadd3B = 8'b00111011;	//Reg Address 0x3B
parameter spidat3B = 8'b01000101;	//End of frame delimit 0x45

//TXB0D6 0x3C Data buffer byte6
parameter spiadd3C = 8'b00111100;	//Reg Address 0x3C
parameter spidat3C = 8'b00000000;	//Dummy byte

//TXB0D7 0x3D Data buffer byte7
parameter spiadd3D = 8'b00111101;	//Reg Address 0x3D
parameter spidat3D = 8'b00000000;	//Dummy byte

//Command in order to send TXB0D0
parameter spicom81 = 8'b10000001;

//****************************//
// 	SPI frame control			//
//****************************//

always @(negedge clk8m)
begin
	if (length_f == 1) begin			//Data length 1byte
		if (framecou == 1) begin		//Before 1byte
			RST <= 0;						//SPI module enable
			framecou <= framecou + 1;
		end
		
		if (framecou == 10) begin		//After 1byte
			RST <= 1;						//SPI module reset
			framecou <= framecou + 1;
		end
		
		//if (framecou == 11) begin
		if (framecou == 3000) begin	//Wait for next CAN data frame
			framecou <= 0;
			addcou <= 9;					//Return to frame No.9
		end
		
		else begin
			framecou <= framecou + 1;
		end
	end
	
	else begin								//Data length 3byte
		if (framecou == 1) begin		//Before 1st byte
			RST <= 0;						//SPI module enable
			framecou <= framecou + 1;
		end
	
		if (framecou == 26) begin		//After 3byte
			RST <= 1;						//SPI module reset
			framecou <= framecou + 1;
		end
	
		if (framecou == 27) begin
			framecou <= 0;					//reset clk8m counter
			addcou <= addcou + 1;
		end
		
		else begin
			framecou <= framecou + 1;
		end
	end
end

//****************************//
//		MCP2515 register set		//
//****************************//

always @(negedge clk8m)
begin
	case(addcou)						//SPI Frame Number

// Config mode(default)
// Register set

		0:
		begin								//CANCTRL MCP2515 Config mode
			length_f <= 0;				//SPI data length 3byte
			spicom <= spicom02;
			spiadd <= spiadd0F;
			spidat <= spidat0FC;
		end

		1:
		begin								//CNF3 in oder to 500Kbps
			spiadd <= spiadd28;
			spidat <= spidat28;
		end
		
		2:
		begin								//CNF2 in oder to 500Kbps
			spiadd <= spiadd29;
			spidat <= spidat29;
		end
		
		3:
		begin								//CANCTRL MCP2515 Normal mode
			spiadd <= spiadd0F;
			spidat <= spidat0F;
		end

// Normal mode
// Register set
		
		4:
		begin								//TXB0SIDH CAN Base ID 10-3 = 0x00
			spiadd <= spiadd31;
			spidat <= spidat31;
		end
		
		5:									//TXB0SIDL CAN Base ID 2-0 = 001
		begin								//Ext ID enable = 1,Ext ID17-16 = 00
			spiadd <= spiadd32;
			spidat <= spidat32;
		end
		
		6:
		begin								//TXB0EID8 CAN Ext ID 15-8 = 0x00
			spiadd <= spiadd33;
			spidat <= spidat33;
		end
		
		7:
		begin								//TXB0EID0 CAN Ext ID 7-0 = 0x00
			spiadd <= spiadd34;
			spidat <= spidat34;
		end
		
		8:
		begin								//TXB0DLC CAN is Data frame. length 8byte
			spiadd <= spiadd35;
			spidat <= spidat35;
		end

//	CAN Data frame
		
		9:
		begin									//Start of frame
			length_f <= 0;
			spicom <= spicom02;
			spiadd <= spiadd36;
			spidat <= rengedt[39:32];
		end
		
		10:									//CAN Data3
		begin
			spiadd <= spiadd37;
			spidat <= rengedt[31:24];
		end
		
		11:									//CAN Data2
		begin
			spiadd <= spiadd38;
			spidat <= rengedt[23:16];
		end
		
		12:									//CAN Data1
		begin
			spiadd <= spiadd39;
			spidat <= rengedt[15:8];
		end
		
		13:									//CAN Data0
		begin
			spiadd <= spiadd3A;
			spidat <= rengedt[7:0];
		end
		
		14:									//End of frame
		begin
			spiadd <= spiadd3B;
			spidat <= spidat3B;
		end
		
		15:									//CAN Dummy byte
		begin
			spiadd <= spiadd3C;
			spidat <= spidat3C;
		end
		
		16:									//CAN dummy byte
		begin
			spiadd <= spiadd3D;
			spidat <= spidat3D;
		end
		
		17:									//Buffer send
		begin
			length_f <= 1;					//SPI data length 1byte
			spicom <= spicom81;
		end
	endcase
end

// SPI module

spi U21(clk8m, RST, length_f, cs1, so1, spicom, spiadd, spidat);

assign cs20 = cs1;						//SPI sleve select
assign so20 = so1;						//SPI MOSI

endmodule
