////////////////////////////////////////////
// 			SPI data transfer					//
//		2013 edition by Mikata Masahito		//
// 	Module Name:    spi 						//
//		Instance Name:  U21						//
////////////////////////////////////////////
// 			Revision record
// 	ver 1.0 New RTL 2014.02.15

module spi(clk8m, RST, length_f, cs1, so1, spicom, spiadd, spidat);

input clk8m;					//SCK GEN
input RST;						//SPI module reset
input length_f;				//SPI data length
input [7:0]spicom;			//MCP2515 command
input [7:0]spiadd;			//MCP2515 register address
input [7:0]spidat;			//MCP2515 register data

output cs1;						//SPI sleve select
output so1;						//SPI MOSI

reg [4:0]gencou;				//clk8m counter
reg cssts;						//SPI sleve select status
reg sodt;						//SPI MOSI status
reg [7:0]spicom_wr;			//SPI data buffer

always @(negedge clk8m)
begin
	if(RST) begin				//SPI module reset RST=1 loop
		gencou <= 0;
		spicom_wr <= spicom;	//SPI 1st byte set
	end
	
	else begin
		if(length_f) begin				//SPI data length 1byte
			case(gencou)
				0:								//clk8m 1st count Data bit7(MSB)
				begin
					gencou <= gencou + 1;
					cssts <= 0;										//CS=0
					sodt <= spicom_wr[7];						//MSB data set
					spicom_wr <= {spicom_wr[6:0], 1'b0};	//Next bit
				end
			
				1,2,3,4,5,6:						//bit6-1
				begin
					gencou <= gencou + 1;
					sodt <= spicom_wr[7];
					spicom_wr <= {spicom_wr[6:0], 1'b0};
				end
				
				7:										//bit0
				begin
					gencou <= gencou + 1;
					sodt <= spicom_wr[7];
				end
				
				8,9,10,11,12:						//After 8count CS=1
				begin
					cssts <= 1;
				end
			endcase
		end
	
		else begin									//SPI data length 3byte
			case(gencou)
				0:										//clk8m 1st count Data bit23(MSB)
				begin
					cssts <= 0;										//CS=0
					gencou <= gencou + 1;
					sodt <= spicom_wr[7];						//MSB data set
					spicom_wr <= {spicom_wr[6:0], 1'b0};	//Next bit
				end
		
				1,2,3,4,5,6,8,9,10,11,12,13,14,16,17,18,19,20,21,22:
				begin
					gencou <= gencou + 1;
					sodt <= spicom_wr[7];
					spicom_wr <= {spicom_wr[6:0], 1'b0};
				end
		
				7:											//Before 8count
				begin
					gencou <= gencou + 1;
					sodt <= spicom_wr[7];
					spicom_wr <= spiadd;				//Next byte register address
				end
		
				15:										//Before 16count
				begin
					gencou <= gencou + 1;
					sodt <= spicom_wr[7];
					spicom_wr <= spidat;				//Next byte register data
				end
			
				23:										//Before 24count
				begin
					gencou <= gencou + 1;
					sodt <= spicom_wr[7];
				end
			
				24,25,26,27,28,29,30:				//After 24count CS=1
				begin
					cssts <= 1;
				end

			endcase
		end
	end
end

// CS,MOSI assign

assign cs1 = cssts;
assign so1 = sodt;

endmodule
